<?xml version="1.0" encoding="iso-8859-1"?>
<!-- $Id: java.bean.xsl,v 1.1 2003/04/05 02:27:22 stephens Exp $ -->
<xsl:transform
  xmlns="http://www.w3.org/html"
  xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
  version="1.1"
>

<!-- 
####################################################################
# JavaBeans support.
#
-->

<xsl:import href="xmi.TaggedValue.xsl" />
<xsl:import href="java.box.xsl" />


<!-- 
If "bean.boundProperty" value
then class needs Bound Property Support.

See http://java.sun.com/docs/books/tutorial/javabeans/properties/bound.html
-->

<xsl:template match="Foundation.Core.Class" mode="java.bean.class.body">
  
  <xsl:variable name="is_bean">
    <xsl:for-each select=".|Foundation.Core.Classifier.feature/Foundation.Core.Attribute">
      <xsl:call-template name="xmi.TaggedValue.inherited.true">
        <xsl:with-param name="tag" select="'bean'" />
      </xsl:call-template>
    </xsl:for-each>
  </xsl:variable>

  <xsl:variable name="is_bound_property">
    <xsl:for-each select=".|Foundation.Core.Classifier.feature/Foundation.Core.Attribute">
      <xsl:call-template name="xmi.TaggedValue.inherited.true">
        <xsl:with-param name="tag" select="'bean.bound'" />
      </xsl:call-template>
    </xsl:for-each>
  </xsl:variable>

  <xsl:variable name="is_vetoable_property">
    <xsl:for-each select=".|Foundation.Core.Classifier.feature/Foundation.Core.Attribute">
      <xsl:call-template name="xmi.TaggedValue.inherited.true">
        <xsl:with-param name="tag" select="'bean.vetoable'" />
      </xsl:call-template>
    </xsl:for-each>
  </xsl:variable>

  <xsl:if test="$is_bound_property or $is_vetoable_property">

  <xsl:text>
/////////////////////////////////////////////////////////
// Bean support
//

</xsl:text>

    </xsl:if>

    <xsl:if test="$is_bound_property">
      private java.beans.PropertyChangeSupport __beanChanges;
      private java.beans.PropertyChangeSupport getPropertyChangeSuppport() { 
        if ( __beanChanges == null ) {
           __beanChanges = new java.beans.PropertyChangeSupport(this);
        }
        return __beanChanges;
      }
      public void addPropertyChangeListener(java.beans.PropertyChangeListener l)
      {
        getPropertyChangeSuppport().addPropertyChangeListener(l);
      }
      public void removePropertyChangeListener(java.beans.PropertyChangeListener l)
      {
        getPropertyChangeSuppport().removePropertyChangeListener(l);
      }
    </xsl:if>


    <xsl:if test="$is_vetoable_property">
      private java.beans.VetoableChangeSupport __beanVetos;
      private java.beans.VetoableChangeSupport getVetoableChangeSupport() { 
        if ( __beanVetos == null ) {
           __beanVetos = new java.beans.VetoableChangeSupport(this);
        }
        return __beanVetos;
      }
      public void addVetoableChangeListener(java.beans.VetoableChangeListener l)
      {
         getVetoableChangeSupport().addVetoableChangeListener(l);
      }
      public void removeVetoableChangeListener(java.beans.VetoableChangeListener l)
      {
         getVetoableChangeSupport().removeVetoableChangeListener(l);
      }
    </xsl:if>

  <xsl:apply-templates select="Foundation.Core.Classifier.feature/Foundation.Core.Attribute" mode="java.bean.class.body" />

</xsl:template>


<!-- 
If the attribute has a tagged value "bean" that is non-null,
then attribute needs JavaBeans accessor methods.

If the "bean" value contains "bound_property"
then attribute setter needs Bound Property Support.

See http://java.sun.com/docs/books/tutorial/javabeans/properties/bound.html
-->

<xsl:template match="Foundation.Core.Attribute" mode="java.bean.class.body">

  <xsl:variable name="is_bean">
    <xsl:call-template name="xmi.TaggedValue.inherited.true">
      <xsl:with-param name="tag" select="'bean'" />
    </xsl:call-template>
  </xsl:variable>

  <xsl:variable name="is_bound_property">
    <xsl:call-template name="xmi.TaggedValue.inherited.true">
      <xsl:with-param name="tag" select="'bean.bound'" />
    </xsl:call-template>
  </xsl:variable>

  <xsl:variable name="is_vetoable_property">
    <xsl:call-template name="xmi.TaggedValue.inherited.true">
      <xsl:with-param name="tag" select="'bean.vetoable'" />
    </xsl:call-template>
  </xsl:variable>

  <xsl:if test="$is_bean or $is_bound_property or $is_vetoable_property">

    <xsl:variable name="java_type">
      <xsl:call-template name="java.type.qualified">
	 <xsl:with-param name="node" select="Foundation.Core.StructuralFeature.type/*" />
        <xsl:with-param name="multi" select="Foundation.Core.StructuralFeature.multiplicity/*" />
      </xsl:call-template>
    </xsl:variable>
    <xsl:variable name="multi_range">
      <xsl:call-template name="xmi.multiplicity.range">
        <xsl:with-param name="node" select="Foundation.Core.StructuralFeature.multiplicity/*" />
      </xsl:call-template>
    </xsl:variable>
    <xsl:variable name="is_indexed" select="$multi_range != '1..1'" />
    <xsl:variable name="java_indexed_type">
      <xsl:call-template name="java.type.qualified">
	 <xsl:with-param name="node" select="Foundation.Core.StructuralFeature.type/*" />
        <xsl:with-param name="multi" select="''" />
      </xsl:call-template>
    </xsl:variable>
    <xsl:variable name="java_attr" select="Foundation.Core.ModelElement.name" />
    <xsl:variable name="java_Attr" 
       select="
	concat(
	  translate(
	    substring($java_attr, 1, 1), 
          'abcdefghijklmnopqrstuvwxyz', 
          'ABCDEFGHIJKLMNOPQRSTUVWXYZ'
         ), 
         substring($java_attr, 2, string-length($java_attr))
	)" />

    <xsl:variable name="firePropChange">
      "<xsl:value-of select="$java_attr" />",
      <xsl:call-template name="java.box">
        <xsl:with-param name="type" select="$java_type" />
        <xsl:with-param name="value" select="'__beanOldValue'" />
      </xsl:call-template>,
      <xsl:call-template name="java.box">
        <xsl:with-param name="type" select="$java_type" />
        <xsl:with-param name="value" select="'__newValue'" />
      </xsl:call-template>
    </xsl:variable>

    <xsl:variable name="firePropChangeIndexed">
      "<xsl:value-of select="$java_attr" />",
      <xsl:call-template name="java.box">
        <xsl:with-param name="type" select="$java_type" />
        <xsl:with-param name="value" select="'__beanOldValue'" />
      </xsl:call-template>,
      <xsl:call-template name="java.box">
        <xsl:with-param name="type" select="$java_indexed_type" />
        <xsl:with-param name="value" select="'__newValue'" />
      </xsl:call-template>
    </xsl:variable>

    <xsl:text>

  //////////////////////////////////////////////////////////
  // JavaBean property "</xsl:text><xsl:value-of select="$java_attr" /><xsl:text>"
  //
</xsl:text>
    <xsl:text>

  // JavaBean property getter method
</xsl:text>
    <xsl:text>  </xsl:text>
    <xsl:text>public </xsl:text>
    <xsl:value-of select="$java_type" /><xsl:text> </xsl:text>
    <xsl:text>get</xsl:text><xsl:value-of select="$java_Attr" />
    <xsl:text>() { return </xsl:text><xsl:value-of select="$java_attr" />
    <xsl:text>; }</xsl:text>
    <xsl:text>

</xsl:text>

    <xsl:if test="$is_indexed">
    <xsl:text>  </xsl:text>
    <xsl:text>public </xsl:text>
    <xsl:value-of select="$java_indexed_type" /><xsl:text> </xsl:text>
    <xsl:text>get</xsl:text><xsl:value-of select="$java_Attr" />
    <xsl:text>(int __index) { return </xsl:text><xsl:value-of select="$java_attr" />[__index];
    <xsl:text> }</xsl:text>
    <xsl:text>

</xsl:text>
    </xsl:if>

    <xsl:text>
  // JavaBean property setter method
</xsl:text>
    <xsl:text>  </xsl:text>
    <xsl:text>public </xsl:text>
    <xsl:text>void </xsl:text>
    <xsl:text>set</xsl:text><xsl:value-of select="$java_Attr" />
    <xsl:text>(</xsl:text><xsl:value-of select="$java_type" /><xsl:text> __newValue) </xsl:text>
    <xsl:if test="$is_vetoable_property">
      <xsl:text>throws java.beans.PropertyVetoException </xsl:text>
    </xsl:if>
    <xsl:text>{ </xsl:text>
    <xsl:if test="$is_bound_property or $is_vetoable_property">
      <xsl:value-of select="$java_type" /> __beanOldValue = <xsl:value-of select="$java_attr" />; 
    </xsl:if>
    <xsl:if test="$is_vetoable_property">
      getVetoableChangeSupport().fireVetoableChange(<xsl:value-of select="$firePropChange" />);
    </xsl:if>
    <xsl:value-of select="$java_attr" /><xsl:text> = __newValue; </xsl:text>
    <xsl:if test="$is_bound_property">
      getPropertyChangeSuppport().firePropertyChange(<xsl:value-of select="$firePropChange" />);
    </xsl:if>
    <xsl:text>}</xsl:text>
    <xsl:text>

</xsl:text>
 
    <xsl:if test="$is_indexed">
    <xsl:text>public </xsl:text>
    <xsl:text>void </xsl:text>
    <xsl:text>set</xsl:text><xsl:value-of select="$java_Attr" />
    <xsl:text>(int __index, </xsl:text><xsl:value-of select="$java_indexed_type" /><xsl:text> __newValue) </xsl:text>
    <xsl:if test="$is_vetoable_property">
      <xsl:text>throws java.beans.PropertyVetoException </xsl:text>
    </xsl:if>
    <xsl:text>{ </xsl:text>
    <xsl:if test="$is_bound_property or $is_vetoable_property">
      <xsl:value-of select="$java_type" /> __beanOldValue = <xsl:value-of select="$java_attr" />; 
    </xsl:if>
    <xsl:if test="$is_vetoable_property">
      getVetoableChangeSupport().fireVetoableChange(<xsl:value-of select="$firePropChangeIndexed" />);
    </xsl:if>
    <xsl:value-of select="$java_attr" /><xsl:text>[__index] = __newValue; </xsl:text>
    <xsl:if test="$is_bound_property">
      getPropertyChangeSuppport().firePropertyChange(<xsl:value-of select="$firePropChangeIndexed" />);
    </xsl:if>
    <xsl:text>}</xsl:text>
    <xsl:text>

</xsl:text>
    </xsl:if>

  </xsl:if>

</xsl:template>


<xsl:template match="*" mode="java.bean.class.body">
</xsl:template>


<xsl:template name="java.bean.class.body">
  <xsl:apply-templates select="." mode="java.bean.class.body" />
</xsl:template>

</xsl:transform>

