<?xml version="1.0" encoding="iso-8859-1"?>
<!-- $Id: java.xsl,v 1.1 2003/04/05 02:27:22 stephens Exp $ -->
<xsl:transform
  xmlns="http://www.w3.org/html"
  xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
  version="1.1"
>

<xsl:import href="xmi.TaggedValue.xsl" />
<xsl:import href="java.bean.xsl" />


<!-- *****************************************x*******************************
  XMI uses "xmi.id" as node id.
  -->

<xsl:key name="xmi.id" match="*" use="@xmi.id" />

<!--
###############################################################
## Package name support.
##
-->

<xsl:template name="java.package">
  <xsl:param name="node" select="." />

  <xsl:call-template name="xmi.package.qualified">
    <xsl:with-param name="node" select="$node" />
    <xsl:with-param name="sep" select="'.'" />
  </xsl:call-template>
</xsl:template>


<!-- #########################################################################
  Return the fully-qualified java package directory for the XMI element.
  -->

<xsl:template name="java.package_dir">
  <xsl:param name="node" select="." />

  <xsl:variable name="d">
    <xsl:call-template name="xmi.package.qualified">
      <xsl:with-param name="node" select="$node" />
      <xsl:with-param name="sep" select="'/'" />
    </xsl:call-template>
  </xsl:variable>

  <xsl:choose>
    <xsl:when test="string-length($d)"><xsl:value-of select="$d" /></xsl:when>
    <xsl:otherwise>.</xsl:otherwise>
  </xsl:choose>

</xsl:template>


<!-- #########################################################################
  Return the fully-qualified java type name for the XMI element.
  -->

<xsl:template match="Foundation.Core.Class|Foundation.Core.Interface|Foundation.Core.DataType" mode="java.type">
  <xsl:param name="multi" select="false()" />
  <xsl:param name="qualified" select="false()" />
  <xsl:param name="encloser" select="false()" />

<!-- -->
  <xsl:message>
java.type ==============================================================
   node = <xsl:value-of select="concat(name(.), '@', generate-id(.))"/>
   name = <xsl:value-of select="Foundation.Core.ModelElement.name" />
   multi = <xsl:value-of select="$multi"/>
   qualified = <xsl:value-of select="$qualified"/>
   encloser = <xsl:value-of select="$encloser"/>
</xsl:message>
<!-- -->

  <xsl:if test="$qualified and not($encloser and generate-id(.) = generate-id($encloser))">

    <xsl:variable name="package">
      <xsl:call-template name="java.package" />
    </xsl:variable>

    <xsl:if test="string-length($package)">
      <xsl:value-of select="$package" /><xsl:text>.</xsl:text>
    </xsl:if>
  </xsl:if>

  <xsl:value-of select="Foundation.Core.ModelElement.name" />

  <xsl:if test="$multi">
    <xsl:variable name="multi_range">
       <xsl:call-template name="xmi.multiplicity.range">
         <xsl:with-param name="node" select="$multi" />
      </xsl:call-template>
    </xsl:variable>

    <xsl:if test="not($multi_range = '' or $multi_range = '0..1' or $multi_range = '1..1')">
      <xsl:text>[]</xsl:text>
    </xsl:if>
  </xsl:if>

</xsl:template>


<xsl:template match="*[@xmi.idref]" mode="java.type">
  <xsl:param name="multi" select="false()" />
  <xsl:param name="qualified" select="false()" />
  <xsl:param name="encloser" select="false()" />

  <xsl:apply-templates select="key('xmi.id', @xmi.idref)" mode="java.type">
    <xsl:with-param name="multi" select="$multi" />
    <xsl:with-param name="qualified" select="$qualified" />
    <xsl:with-param name="encloser" select="$encloser" />
  </xsl:apply-templates>

</xsl:template>


<xsl:template match="*" mode="java.type">
</xsl:template>


<xsl:template name="java.type">
  <xsl:param name="node" select="." />
  <xsl:param name="multi" select="false()" />
  <xsl:param name="qualified" select="true()" />
  <xsl:param name="encloser" select="false()" />
 
<!-- -->
  <xsl:message>
java.type ==============================================================
   node = <xsl:value-of select="concat(name($node), '@', generate-id($node))"/>
   idref = <xsl:value-of select="$node/@xmi.idref" />
   multi = <xsl:value-of select="$multi"/>
   qualified = <xsl:value-of select="$qualified"/>
   encloser = <xsl:value-of select="$encloser"/>
</xsl:message>
<!-- -->

  <xsl:apply-templates select="$node" mode="java.type">
    <xsl:with-param name="multi" select="$multi" />
    <xsl:with-param name="qualified" select="$qualified" />
    <xsl:with-param name="encloser" select="$encloser" />
  </xsl:apply-templates>
</xsl:template>


<xsl:template name="java.type.qualified">
  <xsl:param name="node" select="." />
  <xsl:param name="multi" select="false()" />
  <xsl:param name="encloser" select="false()" />
 
<!-- -->
  <xsl:message>
java.type.qualified ==============================================================
   node = <xsl:value-of select="concat(name($node), '@', generate-id($node))"/>
   idref = <xsl:value-of select="$node/@xmi.idref" />
   multi = <xsl:value-of select="$multi"/>
   qualified = <xsl:value-of select="true()"/>
   encloser = <xsl:value-of select="$encloser"/>
</xsl:message>
<!-- -->

  <xsl:call-template name="java.type">
    <xsl:with-param name="node" select="$node" />
    <xsl:with-param name="multi" select="$multi" />
    <xsl:with-param name="qualified" select="true()" />
    <xsl:with-param name="encloser" select="$encloser" />
  </xsl:call-template>
</xsl:template>


<!-- #########################################################################
  Outputs the Java ownerScope for a node.
  -->

<xsl:template name="java.ownerScope">
  <xsl:param name="node" select="." />

  <xsl:variable name="ownerScope" select="$node/Foundation.Core.Feature.ownerScope/@xmi.value" />

  <xsl:if test="$ownerScope = 'classifier'">
    <xsl:text>static </xsl:text>
  </xsl:if>

</xsl:template>

<!-- #########################################################################
  Outputs the Java visibility modifiers for a node.
  -->

<xsl:template name="java.visibility">
  <xsl:param name="node" select="." />

  <xsl:variable name="visibility" select="$node/Foundation.Core.ModelElement.visibility/@xmi.value" />
  <xsl:variable name="changeability" select="$node/Foundation.Core.StructuralFeature.changeability/@xmi.value" />

  <xsl:if test="string-length($visibility)">
    <xsl:value-of select="$visibility" /><xsl:text> </xsl:text>
  </xsl:if>

  <xsl:if test="$changeability='frozen'">
    <xsl:text>final </xsl:text>
  </xsl:if>

</xsl:template>


<!-- #########################################################################
  Outputs the Java modifiers for a node.
  -->

<xsl:template name="java.modifiers">
  <xsl:param name="node" select="." />

  <xsl:for-each select="$node">
    <xsl:call-template name="java.visibility" />
    <xsl:call-template name="java.ownerScope" />
  </xsl:for-each>

</xsl:template>


<!-- 
######################################################################
# Outputs a java class or interface from a <Foundation.Core.Class> or <Foundation.Core.Interface>
# node.
#
-->

<xsl:template name="java.class_or_interface">
  <xsl:param name="kind" select="'class'" />

  <xsl:variable name="class_id" select="@xmi.id" />
  <xsl:variable name="generate">
    <xsl:call-template name="xmi.TaggedValue.inherited">
      <xsl:with-param name="tag" select="'xmi2code'" />
    </xsl:call-template>
  </xsl:variable>

  <xsl:if test="$generate = 'true'">

  <!-- 
<xsl:copy-of select="."/>
  -->
  <xsl:variable name="package">
    <xsl:call-template name="java.package" />
  </xsl:variable>
  <xsl:variable name="package_dir">
    <xsl:call-template name="java.package_dir" />
  </xsl:variable>
  <xsl:variable name="file_suffix">
    <xsl:call-template name="xmi.TaggedValue.inherited">
      <xsl:with-param name="tag" select="'code.suffix'" />
      <xsl:with-param name="default" select="'.java'" />
    </xsl:call-template>    
  </xsl:variable>

  <xsl:variable name="class_name" select="Foundation.Core.ModelElement.name" />
  <xsl:variable name="class_path" select="concat($package,'.',$class_name)" />
  <xsl:variable name="class_file" select="concat($package_dir,'/',$class_name,$file_suffix)" />

  <!-- #############################################################################
       # File header.
       #
  -->

  <xsl:text>//-// FILE BEGIN </xsl:text><xsl:value-of select="$class_file" /><xsl:text>
</xsl:text>

  <xsl:text>//-// xmi.id="</xsl:text><xsl:value-of select="@xmi.id" /><xsl:text>"
</xsl:text>

  <xsl:if test="string-length($package)">
    <xsl:text>package </xsl:text><xsl:value-of select="$package" /><xsl:text>;
</xsl:text>
  </xsl:if>

  <!-- #############################################################################
       # imports.
       #
  -->

  <!-- #############################################################################
       # class decl.
       #
  -->

  <xsl:call-template name="java.modifiers" />
  <xsl:value-of select="$kind" /><xsl:text> </xsl:text><xsl:value-of select="$class_name" /><xsl:text>
</xsl:text>

  <!-- #############################################################################
       # extends.
       #
  -->
  <xsl:variable
	name="extends"
       select="//Foundation.Core.Generalization[Foundation.Core.Generalization.child/Foundation.Core.GeneralizableElement/@xmi.idref=$class_id]"
  />


  <xsl:if test="$extends">
    <xsl:text>  extends </xsl:text>

<!--
    <xsl:copy-of select="$extends" />
-->
    <xsl:for-each select="$extends">
      <xsl:call-template name="java.type.qualified">
        <xsl:with-param name="node" select="Foundation.Core.Generalization.parent/Foundation.Core.GeneralizableElement" />
      </xsl:call-template>
    </xsl:for-each>
    <xsl:text>
</xsl:text>  
  </xsl:if>


  <!-- #############################################################################
       # implements.
       #
  -->

  <xsl:variable
	name="implements"
       select="//Foundation.Core.Abstraction[Foundation.Core.Dependency.client/Foundation.Core.ModelElement/@xmi.idref=$class_id]"
  />


  <xsl:if test="$implements">
    <xsl:text>  implements </xsl:text>

<!--
    <xsl:copy-of select="$implements" />
-->
    <xsl:for-each select="$implements">
      <xsl:call-template name="java.type.qualified">
        <xsl:with-param name="node" select="Foundation.Core.Dependency.supplier/Foundation.Core.ModelElement" />
      </xsl:call-template>
    </xsl:for-each>
    <xsl:text>
</xsl:text>  
  </xsl:if>



  <!-- #############################################################################
       # instantiates.
       #
  -->

<xsl:variable name="instantiates">
  <xsl:call-template name="xmi.TaggedValue.inherited">
    <xsl:with-param name="tag" select="'instantiates'" />
  </xsl:call-template>
</xsl:variable>
  <xsl:if test="string-length($instantiates)">
  <xsl:text>  instantiates </xsl:text><xsl:value-of select="$instantiates" /><xsl:text>
</xsl:text>
  </xsl:if>

  <!-- #############################################################################
       # Class body.
       #
  -->
<xsl:text>{
</xsl:text>

<xsl:if test="$kind != 'interface'">

  <xsl:text>

/////////////////////////////////////////////////////////
// Attributes
//

</xsl:text>
  <xsl:apply-templates select="Foundation.Core.Classifier.feature/Foundation.Core.Attribute" mode="java" />

  <xsl:text>

/////////////////////////////////////////////////////////
// Associations
//

</xsl:text>
  <xsl:call-template name="java.class_associations" />

</xsl:if>

  <xsl:text>
/////////////////////////////////////////////////////////
// Operations
//

</xsl:text>
  <xsl:for-each select="Foundation.Core.Classifier.feature/Foundation.Core.Operation">
    <xsl:call-template name="java.operation">
      <xsl:with-param name="class_kind" select="$kind" />
    </xsl:call-template>
  </xsl:for-each>


   <!-- JAVA BEANS -->
   <xsl:call-template name="java.bean.class.body" />

  <xsl:text>
}
</xsl:text>

  <xsl:text>//-// FILE END </xsl:text><xsl:value-of select="$class_file" /><xsl:text>
</xsl:text>


</xsl:if>

</xsl:template>



<!-- ************************************************************************
  Foundation.Core.Attribute
  -->

<xsl:template match="Foundation.Core.Attribute" mode="java">

  <xsl:message>attribute owner <xsl:copy-of select="Foundation.Core.Feature.owner/*" /></xsl:message>

  <xsl:variable name="java_type">
    <xsl:call-template name="java.type.qualified">
      <xsl:with-param name="node" select="Foundation.Core.StructuralFeature.type/*" />
      <xsl:with-param name="multi" select="Foundation.Core.StructuralFeature.multiplicity/*" />
      <xsl:with-param name="encloser" select="Foundation.Core.Feature.owner/*" />
    </xsl:call-template>
  </xsl:variable>
  <xsl:variable name="java_name" select="Foundation.Core.ModelElement.name" />
  <xsl:variable name="initial_value" select="Foundation.Core.Attribute.initialValue/Foundation.Data_Types.Expression[Foundation.Data_Types.Expression.language='Java']" />

    <xsl:variable name="multi_range">
      <xsl:call-template name="xmi.multiplicity.range">
        <xsl:with-param name="node" select="Foundation.Core.StructuralFeature.multiplicity/*" />
      </xsl:call-template>
    </xsl:variable>

  <xsl:text>  </xsl:text>

  <xsl:call-template name="java.modifiers" />

  <xsl:value-of select="$java_type" /><xsl:text> </xsl:text>
  <xsl:value-of select="$java_name" />

  <xsl:if test="$initial_value">
    <xsl:text> = </xsl:text>
    <xsl:value-of select="$initial_value/Foundation.Data_Types.Expression.body" />
  </xsl:if>

  <xsl:text>;
</xsl:text>

    <xsl:text>  /* multiplicity = </xsl:text><xsl:value-of select="$multi_range" /><xsl:text>  */
</xsl:text>

</xsl:template>


<!-- ************************************************************************
  Foundation.Core.Operation
  -->

<xsl:template name="java.operation">
  <xsl:param name="class_kind" select="'class'" />
  <xsl:param name="owning_class" select="../.." />

  <xsl:message>owning_class <xsl:value-of select="$owning_class/Foundation.Core.ModelElement.name" /></xsl:message>

  <xsl:variable name="class_name" select="$owning_class/Foundation.Core.ModelElement.name" />
  <xsl:variable name="feature_name" select="Foundation.Core.ModelElement.name" />

  <xsl:variable name="is_create" select="$feature_name = 'new'" />
  <xsl:variable name="is_abstract" select="Foundation.Core.Operation.isAbstract/@xmi.value!='false'" />

  <xsl:variable name="return_param" select="Foundation.Core.BehavioralFeature.parameter/Foundation.Core.Parameter[Foundation.Core.Parameter.kind/@xmi.value='return']" />

  <xsl:variable name="return_type">
    <xsl:choose>
      <xsl:when test="$is_create"></xsl:when>
      <xsl:otherwise>
        <xsl:for-each select="$return_param">
          <xsl:call-template name="java.parameter.type">
            <xsl:with-param name="owning_class" select="$owning_class"/>
          </xsl:call-template>
        </xsl:for-each>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:variable>

  <xsl:variable name="method_name">
    <xsl:choose>
      <xsl:when test="$is_create"><xsl:value-of select="$class_name" /></xsl:when>
      <xsl:otherwise><xsl:value-of select="$feature_name" /></xsl:otherwise>
    </xsl:choose>
  </xsl:variable>

  <!-- MODIFIERS -->

  <xsl:text>  </xsl:text>

  <xsl:call-template name="java.modifiers" />

  <xsl:value-of select="$return_type" /><xsl:text> </xsl:text>
  <xsl:value-of select="$method_name" />
  <xsl:text>(</xsl:text>

  <!-- PARAMETERS -->

  <xsl:variable name="parameters" select="Foundation.Core.BehavioralFeature.parameter/Foundation.Core.Parameter[Foundation.Core.Parameter.kind/@xmi.value != 'return']" />

  <xsl:for-each select="$parameters">
    <xsl:call-template name="java.parameter" />
    <xsl:if test="position() != last()">
      <xsl:text>, </xsl:text>
    </xsl:if>
  </xsl:for-each>
  <xsl:text>)</xsl:text>

  <!-- BODY -->

  <xsl:choose>
    <xsl:when test="$is_abstract or $class_kind = 'interface'">
      <xsl:text>;
</xsl:text>
    </xsl:when>
    <xsl:otherwise>
      <xsl:text>
  {
    </xsl:text>
      <xsl:variable 
        name="specified_body" 
        select="normalize-space(Foundation.Core.ModelElement.taggedValue/Foundation.Extension_Mechanisms.TaggedValue[Foundation.Extension_Mechanisms.TaggedValue.tag='body']/Foundation.Extension_Mechanisms.TaggedValue.value)"
      />
      <xsl:variable name="body">
        <xsl:choose>
          <xsl:when test="$specified_body"><xsl:value-of select="$specified_body"/></xsl:when>
          <xsl:when test="$return_type = 'void'"></xsl:when>
          <xsl:when test="$return_type = 'boolean'">return false;</xsl:when>
          <xsl:when test="$return_type = 'byte'">return (byte) 0;</xsl:when>
          <xsl:when test="$return_type = 'short'">return (short) 0;</xsl:when>
          <xsl:when test="$return_type = 'int'">return (int) 0;</xsl:when>
          <xsl:when test="$return_type = 'long'">return (long) 0;</xsl:when>
          <xsl:when test="$return_type = 'float'">return (float) 0;</xsl:when>
          <xsl:when test="$return_type = 'double'">return (double) 0;</xsl:when>
          <xsl:otherwise>return null;</xsl:otherwise>
        </xsl:choose>
      </xsl:variable>
      <xsl:value-of select="$body" />
      <xsl:text>
  }

</xsl:text>
    </xsl:otherwise>
  </xsl:choose>

</xsl:template>


<!-- ************************************************************************
  Foundation.Core.Parameter
  -->

<!-- Returns the parameter type normalized for the owning class context. -->
<xsl:template name="java.parameter.type">

  <xsl:message>parameter owner <xsl:copy-of select="../../Foundation.Core.Feature.owner/*" /></xsl:message>

  <xsl:variable name="param_type">
    <xsl:call-template name="java.type.qualified">
      <xsl:with-param name="node" select="Foundation.Core.Parameter.type/*" />
      <xsl:with-param name="multi" select="Foundation.Core.Parameter.mulitiplicity/*" />
      <xsl:with-param name="encloser" select="../../Foundation.Core.Feature.owner/*" />
    </xsl:call-template>
  </xsl:variable>
  <xsl:variable name="param_name" select="Foundation.Core.ModelElement.name" />

  <xsl:message>owning_class <xsl:value-of select="key('xmi.id', ../../Foundation.Core.Feature.owner)/Foundation.Core.ModelElement.name" /></xsl:message>
  <xsl:message>operation <xsl:value-of select="../../Foundation.Core.ModelElement.name" /></xsl:message>
  <xsl:message>parameter <xsl:value-of select="Foundation.Core.ModelElement.name" /></xsl:message>
  <xsl:message>param_type <xsl:value-of select="$param_type" /></xsl:message>

  <xsl:value-of select="$param_type" />
</xsl:template>


<xsl:template name="java.parameter">
  <xsl:variable name="param_name" select="Foundation.Core.ModelElement.name" />

  <xsl:variable name="param_type">
    <xsl:call-template name="java.parameter.type">
    </xsl:call-template>
  </xsl:variable>

  <xsl:text></xsl:text><xsl:value-of select="$param_type" />
  <xsl:text> </xsl:text><xsl:value-of select="$param_name" />

</xsl:template>


<!-- #########################################################################
  Outputs the Java attributes and methods for all associations with end points that 
  are attached to this node.
  -->

<xsl:template name="java.class_associations">
  <xsl:param name="class" select="." />
  
  <xsl:variable name="node_id" select="$class/@xmi.id" />

  <xsl:for-each select="//Foundation.Core.Association[Foundation.Core.Association.connection/Foundation.Core.AssociationEnd/Foundation.Core.AssociationEnd.type/Foundation.Core.Classifier/@xmi.idref=$node_id]/Foundation.Core.Association.connection/Foundation.Core.AssociationEnd[Foundation.Core.AssociationEnd.type/Foundation.Core.Classifier/@xmi.idref!=$node_id]">
    <xsl:call-template name="java.class_association_end" />
  </xsl:for-each>

</xsl:template>


<!-- #########################################################################
  Outputs the Java attribute for the association end.
  -->

<xsl:template name="java.class_association_end">

  <xsl:variable name="is_navigable" select="Foundation.Core.AssociationEnd.isNavigable/@xmi.value!='false'" />
<!--
  <xsl:variable name="is_navigable" select="'1'" />
-->


 
<!--
    <xsl:message>
assoc_end = <xsl:copy-of select="." />
is_navigable = <xsl:copy-of select="Foundation.Core.AssociationEnd.isNavigable" />
is_navigable = <xsl:copy-of select="$is_navigable" />
</xsl:message>
-->

  <xsl:if test="$is_navigable">

    <xsl:variable name="assoc" select="../.." />  
    <xsl:variable name="assoc_name" select="$assoc/Foundation.Core.ModelElement.name" />
    <xsl:variable name="end_name" select="Foundation.Core.ModelElement.name" />
    <xsl:variable name="End_name" 
       select="
	concat(
	  translate(
	    substring($end_name, 1, 1), 
          'abcdefghijklmnopqrstuvwxyz', 
          'ABCDEFGHIJKLMNOPQRSTUVWXYZ'
         ), 
         substring($end_name, 2, string-length($end_name))
	)" />

    <xsl:variable name="end_type">
      <xsl:call-template name="java.type.qualified">
	  <xsl:with-param name="node" select="Foundation.Core.AssociationEnd.type/*" />
         <xsl:with-param name="multi" select="Foundation.Core.AssociationEnd.multiplicity/*" />
      </xsl:call-template>
    </xsl:variable>

    <xsl:variable name="multi_range">
      <xsl:call-template name="xmi.multiplicity.range">
        <xsl:with-param name="node" select="Foundation.Core.AssociationEnd.multiplicity/*" />
      </xsl:call-template>
    </xsl:variable>

    <xsl:variable name="java_attr" select="concat($assoc_name, '$', $end_name)" />

    <xsl:variable name="needs_collection" select="not($multi_range = '0..1' or $multi_range = '1..1')" />

    <xsl:variable name="java_collection" select="'java.util.Vector'" />

    <xsl:variable name="java_type">
      <xsl:choose>
        <xsl:when test="$needs_collection"><xsl:value-of select="$java_collection" /></xsl:when>
        <xsl:otherwise><xsl:value-of select="$end_type" /></xsl:otherwise>
      </xsl:choose>
    </xsl:variable>

    <xsl:variable name="java_init_value">
      <xsl:choose>
        <xsl:when test="$needs_collection"> = new <xsl:value-of select="$java_collection" />()</xsl:when>
        <xsl:otherwise></xsl:otherwise>
      </xsl:choose>
    </xsl:variable>

    <xsl:text>  private</xsl:text>
    <xsl:call-template name="java.ownerScope" />
    <xsl:text> </xsl:text><xsl:value-of select="$java_type" />
    <xsl:text> </xsl:text><xsl:value-of select="$java_attr" />
    <xsl:value-of select="$java_init_value" />
    <xsl:text>;
</xsl:text>
    <xsl:text>  /* multiplicity = </xsl:text><xsl:copy-of select="$multi_range" /><xsl:text>  */
</xsl:text>

    <xsl:choose>
      <xsl:when test="$needs_collection">
          <xsl:text>  /* type = </xsl:text><xsl:copy-of select="$end_type" /><xsl:text>  */
</xsl:text>

          <xsl:text>  </xsl:text>
          <xsl:call-template name="java.modifiers" />
          <xsl:text> void</xsl:text>
          <xsl:text> set</xsl:text><xsl:value-of select="$End_name" />
          <xsl:text>(java.lang.Collection __c) { </xsl:text>
          <xsl:value-of select="$java_attr" /><xsl:text> = new </xsl:text>
          <xsl:value-of select="$java_type" /><xsl:text>(__c);</xsl:text>
          <xsl:text> }</xsl:text>
          <xsl:text>

</xsl:text>
 
          <xsl:text>  </xsl:text>
          <xsl:call-template name="java.modifiers" />
          <xsl:text> java.util.Enumeration</xsl:text>
          <xsl:text> get</xsl:text><xsl:value-of select="$End_name" />
          <xsl:text>() { return </xsl:text><xsl:value-of select="$java_attr" /><xsl:text>.elements();</xsl:text>
          <xsl:text> }</xsl:text>
          <xsl:text>

</xsl:text>

          <xsl:text>  </xsl:text>
          <xsl:call-template name="java.modifiers" />
          <xsl:text> void</xsl:text>
          <xsl:text> add</xsl:text><xsl:value-of select="$End_name" />
          <xsl:text>(</xsl:text><xsl:value-of select="$end_type" /><xsl:text> x) { </xsl:text>
          <xsl:text></xsl:text><xsl:value-of select="$java_attr" /><xsl:text>.addElement(x);</xsl:text>
          <xsl:text> }</xsl:text>
          <xsl:text>

</xsl:text>
 
          <xsl:text>  </xsl:text>
          <xsl:call-template name="java.modifiers" />
          <xsl:text> void</xsl:text>
          <xsl:text> remove</xsl:text><xsl:value-of select="$End_name" />
          <xsl:text>(</xsl:text><xsl:value-of select="$end_type" /><xsl:text> x) { </xsl:text>
          <xsl:value-of select="$java_attr" /><xsl:text>.removeElement(x);</xsl:text>
          <xsl:text> }</xsl:text>
          <xsl:text>

</xsl:text>
      </xsl:when>
      <xsl:otherwise>
          <xsl:text>  </xsl:text>
          <xsl:call-template name="java.modifiers" />
          <xsl:text> void</xsl:text>
          <xsl:text> set</xsl:text><xsl:value-of select="$End_name" />
          <xsl:text>(</xsl:text><xsl:value-of select="$java_type" /><xsl:text> x) { </xsl:text>
          <xsl:value-of select="$java_attr" /><xsl:text> = x; </xsl:text>
          <xsl:text>}</xsl:text>
          <xsl:text>

</xsl:text>
 
          <xsl:text>  </xsl:text>
          <xsl:call-template name="java.modifiers" />
          <xsl:text> </xsl:text><xsl:value-of select="$java_type" />
          <xsl:text> get</xsl:text><xsl:value-of select="$End_name" />
          <xsl:text>() { return </xsl:text><xsl:value-of select="$java_attr" />
          <xsl:text>; }</xsl:text>
          <xsl:text>

</xsl:text>
      </xsl:otherwise>
    </xsl:choose>

    <xsl:text>

</xsl:text>
  </xsl:if>

</xsl:template>


<!--
##############################################################################
## Top-level translations
##
-->

<!-- *************************************************************************
  Model_Management.Model
  -->


<xsl:template match="Model_Management.Model" mode="java">
  <xsl:variable name="model_name" select="Foundation.Core.ModelElement.name" />

  <xsl:text>//-// MODEL BEGIN </xsl:text><xsl:value-of select="$model_name" /><xsl:text>
</xsl:text>
  <xsl:text>//-// xmi.id="</xsl:text><xsl:value-of select="@xmi.id" /><xsl:text>"
</xsl:text>

  <xsl:apply-templates select="Foundation.Core.Namespace.ownedElement" mode="java"/>

  <xsl:text>//-// MODEL END </xsl:text><xsl:value-of select="$model_name" /><xsl:text>
</xsl:text>

</xsl:template>


<!-- *************************************************************************
  Foundation.Core.Namespace.ownedElement
  -->

<xsl:template match="Foundation.Core.Namespace.ownedElement" mode="java">
  <xsl:apply-templates select="Foundation.Core.Class" mode="java" />
  <xsl:apply-templates select="Foundation.Core.Interface" mode="java" />
  <xsl:apply-templates select="Model_Management.Package" mode="java" />
</xsl:template>


<!-- *************************************************************************
  Model_Management.Package
  -->

<xsl:template match="Model_Management.Package" mode="java" >

  <xsl:variable name="package_path">
    <xsl:call-template name="java.package" />
  </xsl:variable>
  <xsl:variable name="package_name" select="Foundation.Core.ModelElement.name" />
  <xsl:variable name="pp" select="concat($package_path, '.', $package_name)" />

  <xsl:text>//-// PACKAGE BEGIN </xsl:text><xsl:value-of select="$pp" /><xsl:text>
</xsl:text>
  <xsl:text>//-// xmi.id="</xsl:text><xsl:value-of select="@xmi.id" /><xsl:text>"
</xsl:text>

  <xsl:apply-templates select="Foundation.Core.Namespace.ownedElement" mode="java" />

  <xsl:text>//-// PACKAGE END </xsl:text><xsl:value-of select="$pp" /><xsl:text> : END
</xsl:text>

</xsl:template>


<!-- *************************************************************************
  Foundation.Core.Class
  -->

<xsl:template match="Foundation.Core.Class" mode="java">
  <xsl:call-template name="java.class_or_interface">
    <xsl:with-param name="kind" select="'class'" />
  </xsl:call-template>
</xsl:template>


<!-- *************************************************************************
  Foundation.Core.Interface
  -->

<xsl:template match="Foundation.Core.Interface" mode="java">
  <xsl:call-template name="java.class_or_interface">
    <xsl:with-param name="kind" select="'interface'" />
  </xsl:call-template>
</xsl:template>


</xsl:transform>
